"""
Module to hold the openapi documentation constants.
"""


ANY_TYPE = [
    {"type": "string"},
    {"type": "number"},
    {"type": "integer"},
    {"type": "boolean"},
    {"type": "array"},
    {"type": "object"},
]
